from slai.model_version import ModelVersion
from slai.model_inputs import ModelInputs
from slai.base_handler import BaseModelHandler
from slai.login import Login
from slai import loaders
from slai.model import Model

__version__ = "0.1.17"


# most used slai actions go here
model = Model
model_version = ModelVersion
loaders = loaders
inputs = ModelInputs
base_handler = BaseModelHandler
login = Login


__all__ = [
    "__version__",
    "model_version",
    "model",
    "inputs",
    "base_handler",
    "loaders",
    "login",
]



class InvalidAWSProfile(Exception):
    pass


class InvalidServicePrefix(Exception):
    pass


class ModelNotFound(Exception):
    pass


class ProjectNotFound(Exception):
    pass


class InvalidRuntimeException(Exception):
    pass


class UnsupportedModelFormat(Exception):
    pass


class NoCredentialsFound(Exception):
    pass


class InvalidNotebookAuthToken(Exception):
    pass


class InvalidCredentials(Exception):
    pass


class UnableToConvertNotebook(Exception):
    pass


class IntegrationError(Exception):
    pass



from slai.modules.parameters import from_config
from slai.modules.runtime import detect_runtime, detect_credentials, ValidRuntimes
from slai.clients.inference import get_inference_client


class Model:
    def __init__(self, *, model_name, project_name, model_version_id=None):
        self.inference_client = get_inference_client(
            model_name=model_name,
            project_name=project_name,
            model_version_id=model_version_id,
        )

    def __call__(self, **inputs):
        return self.inference_client.call(payload=inputs)

    def info(self):
        pass



from slai.exceptions import NoCredentialsFound
import requests

from requests.auth import HTTPBasicAuth
from importlib import import_module

from slai.modules.parameters import from_config
from slai.modules.runtime import detect_runtime, detect_credentials

REQUESTS_TIMEOUT = 15


def get_identity_client(*, client_id=None, client_secret=None):
    import_path = from_config(
        "IDENTITY_CLIENT",
        "slai.clients.identity.IdentityClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])
    return getattr(import_module(path), class_)(
        client_id=client_id, client_secret=client_secret
    )


class IdentityClient:
    STAGE = from_config(
        key="STAGE",
        default="staging",
    )

    BASE_URL = from_config(
        key="BASE_URL",
        default=f"https://api.slai.io/{STAGE}",
    )

    def __init__(self, client_id=None, client_secret=None):
        self.runtime = detect_runtime()
        self.credentials_loaded = False

        if not client_id or not client_secret:
            try:
                self.credentials = detect_credentials(runtime=self.runtime)
                self.client_id = self.credentials["client_id"]
                self.client_secret = self.credentials["client_secret"]
                self.credentials_loaded = True
            except NoCredentialsFound:
                pass

        else:
            self.client_id = client_id
            self.client_secret = client_secret

            self.credentials_loaded = True

    def validate_notebook_auth_token(self, *, token):
        body = {"action": "retrieve", "token": token}

        res = requests.post(
            f"{self.BASE_URL}/app/notebook-auth",
            auth=None,
            headers={},
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def get_user(self):
        if not self.credentials_loaded:
            raise NoCredentialsFound("no_credentials_loaded")

        body = {"action": "retrieve"}

        res = requests.post(
            f"{self.BASE_URL}/cli/user",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={},
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()



from slai.clients.project import get_project_client
from slai.clients.cli import get_cli_client
from slai.modules.parameters import from_config
from importlib import import_module


def get_model_client(*, model_name, project_name):
    import_path = from_config(
        "MODEL_CLIENT",
        "slai.clients.model.ModelClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])
    return getattr(import_module(path), class_)(
        model_name=model_name, project_name=project_name
    )


class ModelClient:
    def __init__(self, *, model_name=None, project_name=None):
        self.project_client = get_project_client(project_name=project_name)
        self.cli_client = get_cli_client()

        self.model_name = model_name
        self.model = self.get_model()

    def get_project_name(self):
        return self.project_client.get_project_name()

    def create_model_artifact(
        self, *, model_data, artifact_type, artifact_requirements, model_version_id=None
    ):
        if model_version_id is None:
            model_version_id = self.model["model_version_id"]

        model_artifact = self.cli_client.create_model_artifact(
            model_version_id=model_version_id,
            model_data=model_data,
            artifact_type=artifact_type,
            artifact_requirements=artifact_requirements,
        )
        return model_artifact

    def create_model_deployment(
        self,
        *,
        model_artifact_id,
        model_handler_data,
        requirements,
    ):
        model_deployment = self.cli_client.create_model_deployment(
            model_artifact_id=model_artifact_id,
            model_handler_data=model_handler_data,
            requirements=requirements,
        )
        return model_deployment

    def update_model(self, *, key, value):
        model_data = self.get_model()
        model_data[key] = value
        model_data = self.cli_client.update_model(model_data=model_data)
        return model_data

    def get_model(self):
        model_data = self.cli_client.retrieve_model(
            project_name=self.project_client.get_project_name(),
            name=self.model_name,
        )
        return model_data

    def get_latest_model_artifact(self, model_version_id=None):
        model_data = self.get_model()

        if model_version_id is None:
            model_version_id = model_data["model_version_id"]

        model_artifact = self.cli_client.retrieve_model_artifact(
            model_version_id=model_version_id,
            model_artifact_id=None,
        )
        return model_artifact


class MockModelClient:
    def __init__(self, *, model_name, project_name):
        pass

    def get_project_name(self):
        return "MOCK_PROJECT_NAME"

    def create_model_artifact(
        self, *, model_data, artifact_type, artifact_requirements
    ):
        return {
            "id": "60329220dd191a68cfb474ec",
            "model_version_id": "60328ce6dd191a6608bb7da9",
            "artifact_type": artifact_type,
            "artifact_requirements": {"torch": "==1.70.0a0"},
            "created": "2021-02-21T12:02:24.503850",
        }

    def update_model(self, *, key, value):
        model_data = {
            "id": "60329220dd191a68cfb474ec",
            "project_name": "something",
            "project_id": "60328ce6dd191a6608bb7da9",
            "name": "something1",
            "notebook_uri": None,
            "model_version_id": "60329220dd191a68cfb474ed",
            "created": "2021-02-21T12:02:24.503850",
            "updated": "2021-02-21T12:02:24.635203",
        }
        model_data[key] = value
        return model_data

    def get_model(self):
        return {
            "id": "60329220dd191a68cfb474ec",
            "project_name": "something",
            "project_id": "60328ce6dd191a6608bb7da9",
            "name": "something1",
            "notebook_uri": None,
            "model_version_id": "60329220dd191a68cfb474ed",
            "created": "2021-02-21T12:02:24.503850",
            "updated": "2021-02-21T12:02:24.635203",
        }
